@@ -3,6 +3,7 @@ import torch
 
 import numpy as np
 
+from tqdm import tqdm
 from typing import Any
 from typing import Dict
 from typing import List
@@ -12,6 +13,7 @@ from typing import Optional
 from typing import Protocol
 from dataclasses import dataclass
 from collections import OrderedDict
+from cftool.misc import print_info
 from cftool.misc import lock_manager
 from cftool.misc import safe_execute
 from cftool.misc import shallow_copy_dict
@@ -30,16 +32,18 @@ from ...types import states_callback_type
 from ...schema import Config
 from ...schema import DLConfig
 from ...schema import InferenceOutputs
+from ...trainer import get_scores
 from ...trainer import get_sorted_checkpoints
 from ...pipeline import _get_requirements
 from ...pipeline import IModifier
 from ...pipeline import DLPipeline
 from ...constants import PREDICTIONS_KEY
+from ...constants import CHECKPOINTS_FOLDER
 from ...data.ml import IMLData
 from ...data.ml import MLCarefreeData
 from ...data.ml import IMLDataProcessor
 from ...data.ml.basic import _InternalCarefreeMLDataProcessor
-from ...misc.internal_.inference import MLInference
+from ...implementations.inference import MLInference
 from ...models.ml.encoders import Encoder
 from ...models.ml.encoders import EncodingSettings
 from ...models.schemas.ml import MLModel
@@ -163,7 +167,9 @@ class MLModifier(IModifier, IMLPipeline):
             self._defaults["output_dim"] = self.output_dim
         self.use_auto_loss = self.loss_name == "auto"
         if self.use_auto_loss:
-            if is_reg:
+            if self.core_name == "ddr":
+                self.loss_name = "ddr"
+            elif is_reg:
                 self.loss_name = "mae"
             else:
                 self.loss_name = "bce" if self.output_dim == 1 else "focal"
@@ -238,6 +244,7 @@ class MLModifier(IModifier, IMLPipeline):
         if (
             self.trainer_config.metric_names is None
             and self.use_auto_loss
+            and self.core_name != "ddr"
             and not self.trainer_config.use_losses_as_metrics
         ):
             if self.is_classification:
@@ -260,8 +267,14 @@ class MLModifier(IModifier, IMLPipeline):
         *,
         build_trainer: bool = True,
         report: Optional[bool] = None,
+        report_folder: Optional[str] = None,
     ) -> None:
-        super().build(data_info, build_trainer=build_trainer, report=report)
+        super().build(
+            data_info,
+            build_trainer=build_trainer,
+            report=report,
+            report_folder=report_folder,
+        )
         self.processor.is_ready = True
 
     # load steps
@@ -410,13 +423,22 @@ class MLPipeline(IMLPipeline, DLPipeline):  # type: ignore
         export_folders: List[str],
         *,
         cuda: Optional[str] = None,
+        num_picked: Optional[Union[int, float]] = None,
         compress: bool = True,
         build_trainer: bool = False,
         report: Optional[bool] = None,
+        report_folder: Optional[str] = None,
         states_callback: states_callback_type = None,
         pre_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
         post_callback: Optional[Callable[[DLPipeline, Dict[str, Any]], None]] = None,
     ) -> "MLPipeline":
+        num_total = num_repeat = len(export_folders)
+        if num_picked is not None:
+            if isinstance(num_picked, float):
+                if num_picked < 0.0 or num_picked > 1.0:
+                    raise ValueError("`num_picked` should ∈ [0, 1] when set to float")
+                num_picked = round(num_total * num_picked)
+            num_repeat = num_picked
         export_folder = export_folders[0]
         base_folder = os.path.dirname(os.path.abspath(export_folder))
         with lock_manager(base_folder, [export_folder]):
@@ -432,11 +454,38 @@ class MLPipeline(IMLPipeline, DLPipeline):  # type: ignore
                 loaded = DataModule.load_info(export_folder, return_bytes=True)
                 data_info = loaded.info
                 m.data_module_bytes = loaded.data_module_bytes
-        m._num_repeat = m.config["num_repeat"] = len(export_folders)
-        m._make_modifier().build(data_info, build_trainer=build_trainer, report=report)
+        m._num_repeat = m.config["num_repeat"] = num_repeat
+        m._make_modifier().build(
+            data_info,
+            build_trainer=build_trainer,
+            report=report,
+            report_folder=report_folder,
+        )
         m.model.to(m.device)
         merged_states: OrderedDict[str, torch.Tensor] = OrderedDict()
-        for i, export_folder in enumerate(export_folders):
+        if num_picked is not None:
+            scores = []
+            for i, folder in enumerate(export_folders):
+                base_folder = os.path.dirname(os.path.abspath(folder))
+                with lock_manager(base_folder, [folder]):
+                    with Saving.compress_loader(folder, compress):
+                        folder_scores = get_scores(folder)
+                        if not folder_scores:
+                            checkpoint_folder = os.path.join(folder, CHECKPOINTS_FOLDER)
+                            folder_scores = get_scores(checkpoint_folder)
+                        if not folder_scores:
+                            raise ValueError(f"cannot find scores in '{folder}'")
+                        scores.append(max(folder_scores.values()))
+            scores_array = np.array(scores)
+            picked_indices = np.argsort(scores)[::-1][:num_picked]
+            export_folders = [export_folders[i] for i in picked_indices]
+            original_score = scores_array.mean().item()
+            picked_score = scores_array[picked_indices].mean().item()
+            print_info(
+                f"picked {num_picked} / {num_total}, "
+                f"score: {original_score} -> {picked_score}"
+            )
+        for i, export_folder in enumerate(tqdm(export_folders, desc="fuse")):
             base_folder = os.path.dirname(os.path.abspath(export_folder))
             with lock_manager(base_folder, [export_folder]):
                 with Saving.compress_loader(export_folder, compress):
