@@ -13,6 +13,7 @@ from typing import List
 from typing import Type
 from typing import Tuple
 from typing import Union
+from typing import TypeVar
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
@@ -189,88 +190,11 @@ def register_ml_data_processor(name: str, *, allow_duplicate: bool = False) -> C
     return IMLDataProcessor.register(name, allow_duplicate=allow_duplicate)
 
 
-class IMLDataset(IDataset, metaclass=ABCMeta):
-    @abstractmethod
-    def __getitem__(self, item: Union[int, List[int], np.ndarray]) -> np_dict_type:
-        pass
-
-
-class IMLLoader(IDataLoader, metaclass=ABCMeta):
-    callback: str
-
-    cursor: int
-    indices: np.ndarray
-
-    data: IMLDataset
-    shuffle: bool
-    shuffle_backup: bool
-    name: Optional[str]
-    batch_size: int
-    use_numpy: bool
-
-    def __init__(
-        self,
-        data: IMLDataset,
-        *,
-        shuffle: bool,
-        name: Optional[str] = None,
-        batch_size: int = 128,
-        sample_weights: Optional[np.ndarray] = None,
-        use_numpy: bool = False,
-    ):
-        if sample_weights is not None and len(data) != len(sample_weights):
-            raise ValueError(
-                f"the number of data samples ({len(data)}) is not identical with "
-                f"the number of sample weights ({len(sample_weights)})"
-            )
-        super().__init__(sample_weights=sample_weights)
-        self.data = data
-        self.shuffle = shuffle
-        self.shuffle_backup = shuffle
-        self.name = name
-        self.batch_size = batch_size
-        self.use_numpy = use_numpy
-
-    def __iter__(self) -> "IMLLoader":
-        self.cursor = 0
-        self.indices = get_weighted_indices(len(self.data), self.sample_weights)
-        if self.shuffle:
-            np.random.shuffle(self.indices)
-        return self
-
-    def __next__(self) -> Union[np_dict_type, tensor_dict_type]:
-        start = self.cursor
-        if start >= len(self.data):
-            raise StopIteration
-        self.cursor += self.batch_size
-        indices = self.indices[start : self.cursor]
-        batch = self.data[indices]
-        batch.setdefault(BATCH_INDICES_KEY, indices)
-        if self.use_numpy:
-            return batch
-        return {k: None if v is None else to_torch(v) for k, v in batch.items()}
-
-    def disable_shuffle(self) -> None:
-        self.shuffle = False
-
-    def recover_shuffle(self) -> None:
-        self.shuffle = self.shuffle_backup
-
-    def copy(self) -> "IMLLoader":
-        return self.__class__(
-            self.data,
-            name=self.name,
-            shuffle=self.shuffle,
-            batch_size=self.batch_size,
-            sample_weights=self.sample_weights,
-        )
-
-
 # internal
 
 
 @IDataset.register("ml")
-class MLDataset(IMLDataset):
+class MLDataset(IDataset, metaclass=ABCMeta):
     def __init__(
         self,
         x: np.ndarray,
@@ -302,12 +226,78 @@ class MLDataset(IMLDataset):
         return batch
 
     def __len__(self) -> int:
-        return self.processor.get_num_samples(self.x, self.tag) or len(self.x)
+        num_processor_samples = self.processor.get_num_samples(self.x, self.tag)
+        return len(self.x) if num_processor_samples is None else num_processor_samples
 
 
 @IDataLoader.register("ml")
-class MLLoader(IMLLoader):
-    pass
+class MLLoader(IDataLoader, metaclass=ABCMeta):
+    callback: str
+
+    cursor: int
+    indices: np.ndarray
+
+    data: MLDataset
+    shuffle: bool
+    shuffle_backup: bool
+    name: Optional[str]
+
+    def __init__(
+        self,
+        data: MLDataset,
+        *,
+        shuffle: bool,
+        name: Optional[str] = None,
+        batch_size: int = 128,
+        sample_weights: Optional[np.ndarray] = None,
+        use_numpy: bool = False,
+    ):
+        if sample_weights is not None and len(data) != len(sample_weights):
+            raise ValueError(
+                f"the number of data samples ({len(data)}) is not identical with "
+                f"the number of sample weights ({len(sample_weights)})"
+            )
+        super().__init__(sample_weights=sample_weights)
+        self.data = data
+        self.shuffle = shuffle
+        self.shuffle_backup = shuffle
+        self.name = name
+        self.batch_size = batch_size
+        self.use_numpy = use_numpy
+
+    def __iter__(self) -> "MLLoader":
+        self.cursor = 0
+        self.indices = get_weighted_indices(len(self.data), self.sample_weights)
+        if self.shuffle:
+            np.random.shuffle(self.indices)
+        return self
+
+    def __next__(self) -> Union[np_dict_type, tensor_dict_type]:
+        start = self.cursor
+        if start >= len(self.data):
+            raise StopIteration
+        self.cursor += self.batch_size
+        indices = self.indices[start : self.cursor]
+        batch = self.data[indices]
+        batch.setdefault(BATCH_INDICES_KEY, indices)
+        if self.use_numpy:
+            return batch
+        return {k: None if v is None else to_torch(v) for k, v in batch.items()}
+
+    def disable_shuffle(self) -> None:
+        self.shuffle = False
+
+    def recover_shuffle(self) -> None:
+        self.shuffle = self.shuffle_backup
+
+    def copy(self) -> "MLLoader":
+        return self.__class__(
+            self.data,
+            name=self.name,
+            shuffle=self.shuffle,
+            batch_size=self.batch_size,
+            sample_weights=self.sample_weights,
+        )
 
 
 class IMLPreProcessedXY(NamedTuple):
@@ -315,6 +305,9 @@ class IMLPreProcessedXY(NamedTuple):
     y: Optional[np.ndarray]
 
 
+TMLData = TypeVar("TMLData", bound="IMLData")
+
+
 class IMLData(DLDataModule, metaclass=ConfigMeta):
     config: Dict[str, Any]
 
@@ -460,13 +453,14 @@ class IMLData(DLDataModule, metaclass=ConfigMeta):
         return {
             "type": self.__identifier__,
             "processor": self.processor,
+            "config": self.config,
             "input_dim": self.input_dim,
             "num_history": self.num_history,
             "num_classes": self.num_classes,
             "is_classification": self.is_classification,
         }
 
-    def prepare(self, sample_weights: sample_weights_type) -> None:
+    def prepare(self: TMLData, sample_weights: sample_weights_type) -> TMLData:
         train_others = self.train_others or {}
         valid_others = self.valid_others or {}
         self.train_weights, self.valid_weights = _split_sw(sample_weights)
@@ -505,6 +499,7 @@ class IMLData(DLDataModule, metaclass=ConfigMeta):
                 MLDatasetTag.VALID,
                 **valid_others,
             )
+        return self
 
     def initialize(self) -> Tuple[MLLoader, Optional[MLLoader]]:
         if self.processor is None:
@@ -930,8 +925,6 @@ __all__ = [
     "IMLBatch",
     "IMLDataProcessor",
     "IMLPreProcessedData",
-    "IMLDataset",
-    "IMLLoader",
     "IMLData",
     "MLDataset",
     "MLDatasetTag",
