@@ -18,12 +18,14 @@ from typing import Protocol
 from typing import NamedTuple
 from collections import Counter
 from dataclasses import dataclass
+from cftool.dist import Parallel
 from cftool.misc import grouped
 from cftool.misc import hash_code
 from cftool.misc import print_info
 from cftool.misc import print_warning
 from cftool.misc import DataClassBase
 from cftool.array import get_unique_indices
+from cftool.array import SharedArray
 from cftool.array import StrideArray
 
 from .basic import register_ml_data
@@ -56,6 +58,7 @@ class TimeSeriesConfig(DataClassBase):
     num_test: Optional[int] = None
     enforce_num_test: bool = False
     enforce_test_valid: bool = False
+    num_workers: Optional[int] = None
     random_sample_ratio: Optional[float] = None
     for_inference: bool = False
     sanity_check: bool = False
@@ -79,6 +82,7 @@ class TimeSeriesConfig(DataClassBase):
     @property
     def hash_excludes(self) -> List[str]:
         return [
+            "num_workers",
             "random_sample_ratio",
             "sanity_check",
             "no_cache",
@@ -199,10 +203,10 @@ class ITimeSeriesProcessor(IMLDataProcessor):
 
     def sanity_check(self, tag: MLDatasetTag, bundle: TimeSeriesDataBundle) -> None:
         def check_ids(data: np.ndarray) -> np.ndarray:
-            ids = data[..., self.config.id_column].astype(int)
-            if np.abs(ids[:, 1:] - ids[:, :-1]).sum().item() != 0:
-                raise ValueError(f"multiple ids occurred in one sample : {ids}")
-            return ids
+            ids_ = data[..., self.config.id_column].astype(int)
+            if np.abs(ids_[:, 1:] - ids_[:, :-1]).sum().item() != 0:
+                raise ValueError(f"multiple ids occurred in one sample : {ids_}")
+            return ids_
 
         def get_time_anchors(data: np.ndarray) -> np.ndarray:
             times = data[..., self.config.time_columns]
@@ -282,6 +286,10 @@ class ITimeSeriesProcessor(IMLDataProcessor):
 
     # utils
 
+    def verbose(self, msg: str) -> None:
+        if self.config.verbose:
+            print_info(msg)
+
     @property
     def hashed_cache_folder(self) -> str:
         return os.path.join(self.config.cache_folder, self.tag, self.config.hash)
@@ -298,24 +306,47 @@ class ITimeSeriesProcessor(IMLDataProcessor):
         return TimeSeriesCachePaths(**paths)
 
     def load_cache_paths(self, paths: TimeSeriesCachePaths) -> TimeSeriesDataBundle:
-        if self.config.verbose:
-            print_info("loading data")
+        self.verbose("loading data")
         data = np.load(paths.data_path)
-        if self.config.verbose:
-            print_info("loading rolled indices")
+        self.verbose("loading rolled indices")
         np_merged_indices = np.load(paths.merged_indices_path)
         rolled_indices = StrideArray(np_merged_indices).roll(self.config.span)
-        if self.config.verbose:
-            print_info("loading split indices")
+        self.verbose("loading split indices")
         with open(paths.split_indices_path, "r") as f:
             split_indices = json.load(f)
-        if self.config.verbose:
-            print_info("loading valid indices")
+        self.verbose("loading valid indices")
         valid_indices = np.load(paths.valid_indices_path)
         # return
         args = self.config.copy(), data, split_indices, rolled_indices, valid_indices
         return TimeSeriesDataBundle(*args)
 
+    def _get_valid_indices(
+        self,
+        i: int,
+        num: int,
+        data: np.ndarray,
+        rolled_indices: np.ndarray,
+        num_indices_cumsum: np.ndarray,
+        boolean_mask: Optional[np.ndarray],
+    ) -> Optional[List[int]]:
+        if num < self.config.span:
+            return [] if boolean_mask is None else None
+        cumulate = 0 if i == 0 else num_indices_cumsum[i - 1]
+        valid_indices: Optional[List[int]] = [] if boolean_mask is None else None
+        for j in range(num - self.config.span + 1):
+            idx = j + cumulate
+            is_valid = False
+            if self.config.num_test is not None and self.config.enforce_test_valid:
+                is_valid = True
+            elif self.check_valid(data[rolled_indices[idx]]):
+                is_valid = True
+            if is_valid:
+                if boolean_mask is not None:
+                    boolean_mask[idx] = True
+                elif valid_indices is not None:
+                    valid_indices.append(idx)
+        return valid_indices
+
     def get_bundle(
         self,
         x_train: np.ndarray,
@@ -327,8 +358,7 @@ class ITimeSeriesProcessor(IMLDataProcessor):
         if not self.config.no_cache:
             np.save(cache_paths.data_path, data)
         # split indices
-        if self.config.verbose:
-            print_info("group by id")
+        self.verbose("group by id")
         ids = data[..., self.config.id_column]
         split_indices = get_unique_indices(ids).split_indices
         split_indices = [indices.tolist() for indices in split_indices]
@@ -343,30 +373,38 @@ class ITimeSeriesProcessor(IMLDataProcessor):
         for indices in split_indices:
             merged_indices += indices
         np_merged_indices = np.array(merged_indices, int)
-        if self.config.verbose:
-            print_info("rolling with window")
         rolled_indices = StrideArray(np_merged_indices).roll(self.config.span)
         if not self.config.no_cache:
             np.save(cache_paths.merged_indices_path, np_merged_indices)
         # valid indices
+        fn = self._get_valid_indices
         num_indices = np.array(list(map(len, split_indices)), int)
         num_indices_cumsum = np.cumsum(num_indices)
-        valid_indices = []
-        for i, num in tqdm(
-            enumerate(num_indices),
-            desc="check_valid",
-            total=len(num_indices),
-        ):
-            if num < self.config.span:
-                continue
-            cumulate = 0 if i == 0 else num_indices_cumsum[i - 1]
-            for j in range(num - self.config.span + 1):
-                idx = j + cumulate
-                if self.config.num_test is not None and self.config.enforce_test_valid:
-                    valid_indices.append(idx)
-                elif self.check_valid(data[rolled_indices[idx]]):
-                    valid_indices.append(idx)
-        valid_indices = np.array(valid_indices, int)
+        if not self.config.num_workers:
+            valid_indices_list: List[int] = []
+            for i, num in enumerate(tqdm(num_indices, desc="check_valid")):
+                i_args = i, num, data, rolled_indices, num_indices_cumsum, None
+                valid_indices_list += fn(*i_args) or []
+            valid_indices = np.array(valid_indices_list, int)
+        else:
+            self.verbose("preparing shared arrays")
+            shared_arrays = (
+                SharedArray.from_data(data),
+                SharedArray.from_data(np_merged_indices),
+                SharedArray.from_data(num_indices_cumsum),
+                SharedArray(np.bool, [num_indices_cumsum[-1]]),
+            )
+            common_args = [shared.value for shared in shared_arrays]
+            common_args[1] = StrideArray(common_args[1]).roll(self.config.span)
+            args_list: List[List[Any]] = [[] for _ in range(6)]
+            for i, num in enumerate(num_indices):
+                for j, arg in enumerate([i, num, *common_args]):
+                    args_list[j].append(arg)
+            self.verbose("launching workers")
+            Parallel(self.config.num_workers).grouped(fn, *args_list)
+            valid_indices = np.nonzero(common_args[-1])[0]
+            for shared in shared_arrays:
+                shared.destroy()
         if not self.config.no_cache:
             np.save(cache_paths.valid_indices_path, valid_indices)
         # random sample
@@ -438,19 +476,16 @@ class ITimeSeriesProcessor(IMLDataProcessor):
             and tr_paths.all_exists
             and (not config.use_validation or cv_paths.all_exists)
         ):
-            if config.verbose:
-                print_info("loading train caches")
+            self.verbose("loading train caches")
             self._train_bundle = self.load_cache_paths(tr_paths)
             if not config.use_validation:
                 self._validation_bundle = None
             else:
-                if config.verbose:
-                    print_info("loading validation caches")
+                self.verbose("loading validation caches")
                 self._validation_bundle = self.load_cache_paths(cv_paths)
         # build
         else:
-            if config.verbose:
-                print_info("sort by time")
+            self.verbose("sort by time")
             time_columns = x_train[..., config.time_columns].astype(int)
             time_anchors = self.get_time_anchors(time_columns).ravel()
             sorted_indices = np.argsort(time_anchors)
@@ -462,21 +497,18 @@ class ITimeSeriesProcessor(IMLDataProcessor):
                     sorted_indices,
                     sorted_anchors,
                 )
-                if config.verbose:
-                    print_info("generating train bundle")
+                self.verbose("generating train bundle")
                 train_bundle = self.get_bundle(x_train, tr_indices, tr_paths)
                 if cv_indices is None:
                     validation_bundle = None
                 else:
-                    if config.verbose:
-                        print_info("generating validation bundle")
+                    self.verbose("generating validation bundle")
                     validation_bundle = self.get_bundle(x_train, cv_indices, cv_paths)
                 self._train_bundle = train_bundle
                 self._validation_bundle = validation_bundle
             else:
                 tag = "test" if config.for_inference else "train"
-                if config.verbose:
-                    print_info(f"generating {tag} bundle")
+                self.verbose(f"generating {tag} bundle")
                 if not config.for_inference:
                     indices = sorted_indices
                 else:
@@ -501,8 +533,7 @@ class ITimeSeriesProcessor(IMLDataProcessor):
                         )
                 self._train_bundle = self.get_bundle(x_train, indices, tr_paths)
                 self._validation_bundle = None
-        if config.verbose:
-            print_info("done")
+        self.verbose("done")
 
     def preprocess(self, config: TimeSeriesConfig) -> IMLPreProcessedData:  # type: ignore
         num_test_enforced = config.num_test is not None and config.enforce_test_valid
@@ -532,19 +563,20 @@ class ITimeSeriesProcessor(IMLDataProcessor):
 
     def loads(self, dumped: Dict[str, Any]) -> None:
         def _load(d: Dict[str, Any], tag: str) -> TimeSeriesDataBundle:
-            if self.config.verbose:
-                print_info(f"loading {tag} caches")
+            self.verbose(f"loading {tag} caches")
             cache_paths = self.cache_paths_base(**d)
             return self.load_cache_paths(cache_paths)
 
         tr_paths_d = dumped.pop("tr_cache_paths")
         cv_paths_d = dumped.pop("cv_cache_paths")
         self.config = self.config_base(**dumped)
+        self.verbose("load begin")
         self._train_bundle = _load(tr_paths_d, "train")
         if cv_paths_d is None:
             self._validation_bundle = None
         else:
             self._validation_bundle = _load(cv_paths_d, "validation")
+        self.verbose("load complete")
 
     # callbacks
 
