@@ -63,7 +63,6 @@ from .misc.toolkit import eval_context
 from .modules.optimizers import optimizer_dict
 from .modules.schedulers import scheduler_dict
 from .modules.schedulers import WarmupScheduler
-from .misc.internal_.monitors import ConservativeMonitor
 
 
 class amp_context(context_error_handler):
@@ -134,14 +133,20 @@ class DefaultOptimizerSettings(NamedTuple):
         )
 
 
+def get_scores(folder: str) -> Dict[str, float]:
+    scores_path = os.path.join(folder, SCORES_FILE)
+    if not os.path.isfile(scores_path):
+        return {}
+    with open(scores_path, "r") as f:
+        return json.load(f)
+
+
 def get_sorted_checkpoints(checkpoint_folder: str) -> List[str]:
     # better checkpoints will be placed earlier,
     #  which means `checkpoints[0]` is the best checkpoint
-    scores_path = os.path.join(checkpoint_folder, SCORES_FILE)
-    if not os.path.isfile(scores_path):
+    scores = get_scores(checkpoint_folder)
+    if not scores:
         return []
-    with open(scores_path, "r") as f:
-        scores = json.load(f)
     return list(sort_dict_by_value(scores, reverse=True).keys())
 
 
@@ -379,7 +384,7 @@ class Trainer(ITrainer):
         assert self.rank is not None
         # monitor
         for monitor in self.monitors:
-            if not isinstance(monitor, ConservativeMonitor):
+            if monitor.__identifier__ != "conservative":
                 raise ValueError("Only `ConservativeMonitor` could be used in ddp mode")
         # ddp setup
         _setup_ddp()
